@@ -20,7 +20,7 @@
 # USA
 
 import sys
-import gobject
+from gi import _gobject
 from gi.repository import GObject
 from ..overrides import override
 from ..importer import modules
@@ -35,6 +35,18 @@ else:
 Gtk = modules['Gtk']._introspection_module
 __all__ = []
 
+if Gtk._version == '2.0':
+    import warnings
+    warn_msg = "You have imported the Gtk 2.0 module.  Because Gtk 2.0 \
+was not designed for use with introspection some of the \
+interfaces and API will fail.  As such this is not supported \
+by the pygobject development team and we encourage you to \
+port your app to Gtk 3 or greater. PyGTK is the recomended \
+python module to use with Gtk 2.0"
+
+    warnings.warn(warn_msg, RuntimeWarning)
+
+
 class Widget(Gtk.Widget):
 
     def translate_coordinates(self, dest_widget, src_x, src_y):
@@ -43,6 +55,9 @@ class Widget(Gtk.Widget):
         if success:
             return (dest_x, dest_y,)
 
+    def render_icon(self, stock_id, size, detail=None):
+        return super(Widget, self).render_icon(stock_id, size, detail)
+
 Widget = override(Widget)
 __all__.append('Widget')
 
@@ -140,7 +155,10 @@ class ActionGroup(Gtk.ActionGroup):
         def _process_action(name, stock_id=None, label=None, accelerator=None, tooltip=None, callback=None):
             action = Action(name, label, tooltip, stock_id)
             if callback is not None:
-                action.connect('activate', callback, user_data)
+                if user_data is None:
+                    action.connect('activate', callback)
+                else:
+                    action.connect('activate', callback, user_data)
 
             self.add_action_with_accel(action, accelerator)
 
@@ -186,7 +204,10 @@ class ActionGroup(Gtk.ActionGroup):
             action = Gtk.ToggleAction(name, label, tooltip, stock_id)
             action.set_active(is_active)
             if callback is not None:
-                action.connect('activate', callback, user_data)
+                if user_data is None:
+                    action.connect('activate', callback)
+                else:
+                    action.connect('activate', callback, user_data)
 
             self.add_action_with_accel(action, accelerator)
 
@@ -254,7 +275,10 @@ class ActionGroup(Gtk.ActionGroup):
                 first_action = action
 
         if first_action is not None and on_change is not None:
-            first_action.connect('changed', on_change, user_data)
+            if user_data is None:
+                action.connect('changed', on_change)
+            else:
+                action.connect('changed', on_change, user_data)
 
 ActionGroup = override(ActionGroup)
 __all__.append('ActionGroup')
@@ -284,6 +308,32 @@ class ComboBox(Gtk.ComboBox, Container):
 ComboBox = override(ComboBox)
 __all__.append('ComboBox')
 
+
+class Box(Gtk.Box):
+    def __init__(self, homogeneous=False, spacing=0, **kwds):
+        super(Box, self).__init__(**kwds)
+        self.set_homogeneous(homogeneous)
+        self.set_spacing(spacing)
+
+Box = override(Box)
+__all__.append('Box')
+
+class SizeGroup(Gtk.SizeGroup):
+    def __init__(self, mode=Gtk.SizeGroupMode.VERTICAL):
+        super(SizeGroup, self).__init__(mode=mode)
+
+SizeGroup = override(SizeGroup)
+__all__.append('SizeGroup')
+
+
+class MenuItem(Gtk.MenuItem):
+    def __init__(self, label=None):
+        super(MenuItem, self).__init__(label=label)
+
+MenuItem = override(MenuItem)
+__all__.append('MenuItem')
+
+
 class Builder(Gtk.Builder):
 
     def connect_signals(self, obj_or_map):
@@ -365,11 +415,15 @@ class Dialog(Gtk.Dialog, Container):
             if flags & Gtk.DialogFlags.NO_SEPARATOR:
                 self.set_has_separator(False)
         except AttributeError:
-            pass
+            import warnings
+            warnings.warn("Gtk.DialogFlags.NO_SEPARATOR has been depricated since Gtk+-3.0", DeprecationWarning)
 
         if buttons is not None:
             self.add_buttons(*buttons)
 
+    action_area = property(lambda dialog: dialog.get_action_area())
+    vbox = property(lambda dialog: dialog.get_content_area())
+
     def add_buttons(self, *args):
         """
         The add_buttons() method adds several buttons to the Gtk.Dialog using
@@ -401,15 +455,23 @@ class MessageDialog(Gtk.MessageDialog, Dialog):
     def __init__(self,
                  parent=None,
                  flags=0,
-                 type=Gtk.MessageType.INFO,
+                 message_type=Gtk.MessageType.INFO,
                  buttons=Gtk.ButtonsType.NONE,
                  message_format=None,
                  **kwds):
 
         if message_format != None:
             kwds['text'] = message_format
+
+        # type keyword is used for backwards compat with PyGTK
+        if 'type' in kwds:
+            import warnings
+            warnings.warn("The use of the keyword type as a parameter of the Gtk.MessageDialog constructor has been depricated. Please use message_type instead.", DeprecationWarning)
+            message_type = kwds.pop('type')
+
         Gtk.MessageDialog.__init__(self,
                                    _buttons_property=buttons,
+                                   message_type=message_type,
                                    **kwds)
         Dialog.__init__(self, parent=parent, flags=flags)
 
@@ -618,12 +680,18 @@ class TextIter(Gtk.TextIter):
     def forward_search(self, string, flags, limit):
         success, match_start, match_end = super(TextIter, self).forward_search(string,
             flags, limit)
-        return (match_start, match_end,)
+        if success:
+            return (match_start, match_end)
+        else:
+            return None
 
     def backward_search(self, string, flags, limit):
         success, match_start, match_end = super(TextIter, self).backward_search(string,
             flags, limit)
-        return (match_start, match_end,)
+        if success:
+            return (match_start, match_end)
+        else:
+            return None
 
     def begins_tag(self, tag=None):
         return super(TextIter, self).begins_tag(tag)
@@ -731,9 +799,7 @@ class TreeModel(Gtk.TreeModel):
 
             # we may need to convert to a basic type
             type_ = self.get_column_type(column)
-            if type_ == GObject.TYPE_PYOBJECT:
-                pass # short-circut branching
-            elif type_ == GObject.TYPE_STRING:
+            if type_ == GObject.TYPE_STRING:
                 if isinstance(value, str):
                     value = str(value)
                 elif sys.version_info < (3, 0):
@@ -801,6 +867,9 @@ class TreeModel(Gtk.TreeModel):
                 elif type_ == GObject.TYPE_UINT64:
                     value_container.set_uint64(value)
                     value = value_container
+                elif type_ == GObject.TYPE_PYOBJECT:
+                    value_container.set_boxed(value)
+                    value = value_container
 
             return value
 
@@ -819,6 +888,9 @@ class TreeModel(Gtk.TreeModel):
 
         return tuple(values)
 
+    def filter_new(self, root=None):
+        return super(TreeModel, self).filter_new(root)
+
 TreeModel = override(TreeModel)
 __all__.append('TreeModel')
 
@@ -877,10 +949,43 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
 
         return treeiter
 
+    def prepend(self, row=None):
+        treeiter = Gtk.ListStore.prepend(self)
+
+        if row is not None:
+            self.set_row(treeiter, row)
+
+        return treeiter
+
     def set_value(self, treeiter, column, value):
         value = self._convert_value(treeiter, column, value)
         Gtk.ListStore.set_value(self, treeiter, column, value)
 
+    def set(self, treeiter, *args):
+
+        def _set_lists(columns, values):
+            if len(columns) != len(values):
+                raise TypeError('The number of columns do not match the number of values')
+            for col_num, val in zip(columns, values):
+                if not isinstance(col_num, int):
+                    raise TypeError('TypeError: Expected integer argument for column.')
+                self.set_value(treeiter, col_num, val)
+
+        if args:
+            if isinstance(args[0], int):
+                columns = args[::2]
+                values = args[1::2]
+                _set_lists(columns, values)
+            elif isinstance(args[0], (tuple, list)):
+                if len(args) != 2:
+                    raise TypeError('Too many arguments');
+                _set_lists(args[0], args[1])
+            elif isinstance(args[0], dict):
+                columns = args[0].keys()
+                values = args[0].values()
+                _set_lists(columns, values)
+            else:
+                raise TypeError('Argument list must be in the form of (column, value, ...), ((columns,...), (values, ...)) or {column: value}.  No -1 termination is needed.')
 
 ListStore = override(ListStore)
 __all__.append('ListStore')
@@ -1055,12 +1160,42 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
         value = self._convert_value(treeiter, column, value)
         Gtk.TreeStore.set_value(self, treeiter, column, value)
 
+    def set(self, treeiter, *args):
+
+        def _set_lists(columns, values):
+            if len(columns) != len(values):
+                raise TypeError('The number of columns do not match the number of values')
+            for col_num, val in zip(columns, values):
+                if not isinstance(col_num, int):
+                    raise TypeError('TypeError: Expected integer argument for column.')
+                self.set_value(treeiter, col_num, val)
+
+        if args:
+            if isinstance(args[0], int):
+                columns = args[::2]
+                values = args[1::2]
+                _set_lists(columns, values)
+            elif isinstance(args[0], (tuple, list)):
+                if len(args) != 2:
+                    raise TypeError('Too many arguments');
+                _set_lists(args[0], args[1])
+            elif isinstance(args[0], dict):
+                columns = args[0].keys()
+                values = args[0].values()
+                _set_lists(columns, values)
+            else:
+                raise TypeError('Argument list must be in the form of (column, value, ...), ((columns,...), (values, ...)) or {column: value}.  No -1 termination is needed.')
 
 TreeStore = override(TreeStore)
 __all__.append('TreeStore')
 
 class TreeView(Gtk.TreeView, Container):
 
+    def __init__(self, model=None):
+        Gtk.TreeView.__init__(self)
+        if model:
+            self.set_model(model)
+
     def get_path_at_pos(self, x, y):
         success, path, column, cell_x, cell_y = super(TreeView, self).get_path_at_pos(x, y)
         if success:
@@ -1116,6 +1251,8 @@ class TreeViewColumn(Gtk.TreeViewColumn):
         if success:
             return (start_pos, width,)
 
+    def set_cell_data_func(self, cell_renderer, func, func_data=None):
+        super(TreeViewColumn, self).set_cell_data_func(cell_renderer, func, func_data)
 
 TreeViewColumn = override(TreeViewColumn)
 __all__.append('TreeViewColumn')
