@@ -26,9 +26,9 @@
 
 #include "pygi-foreign.h"
 
-PyObject *g_variant_to_arg(PyObject       *value,
-                           GITypeInfo     *type_info,
-                           GITransfer      transfer,
+PyObject *g_variant_to_arg(PyObject        *value,
+                           GIInterfaceInfo *interface_info,
+                           GITransfer       transfer,
                            GIArgument      *arg);
 
 PyObject *g_variant_from_arg(GITypeInfo *type_info,
