@@ -190,6 +190,21 @@ class TestGLib(unittest.TestCase):
         self.assertEqual(variant.get_type_string(), '(ia{sa{sb}}s)')
         self.assertEqual(variant.unpack(), obj)
 
+        obj = {"frequency": GLib.Variant('t', 738000000),
+            "hierarchy": GLib.Variant('i', 0),
+            "bandwidth": GLib.Variant('x', 8),
+            "code-rate-hp": GLib.Variant('d', 2.0/3.0),
+            "constellation": GLib.Variant('s', "QAM16"),
+            "guard-interval": GLib.Variant('u', 4),}
+        variant = GLib.Variant('a{sv}', obj)
+        self.assertEqual(variant.get_type_string(), 'a{sv}')
+        self.assertEqual(variant.unpack(), {"frequency": 738000000,
+            "hierarchy": 0,
+            "bandwidth": 8,
+            "code-rate-hp": 2.0/3.0,
+            "constellation": "QAM16",
+            "guard-interval": 4})
+
     def test_gvariant_create_errors(self):
         # excess arguments
         self.assertRaises(TypeError, GLib.Variant, 'i', 42, 3)
@@ -233,8 +248,7 @@ class TestGLib(unittest.TestCase):
         self.assertEqual(res, (-1, 'hello'))
 
         # array
-        vb = GLib.VariantBuilder()
-        vb.init(gi._gi.variant_type_from_string('ai'))
+        vb = GLib.VariantBuilder.new(gi._gi.variant_type_from_string('ai'))
         vb.add_value(GLib.Variant.new_int32(-1))
         vb.add_value(GLib.Variant.new_int32(3))
         res = vb.end().unpack()
@@ -246,8 +260,7 @@ class TestGLib(unittest.TestCase):
 
     def test_gvariant_iteration(self):
         # array index access
-        vb = GLib.VariantBuilder()
-        vb.init(gi._gi.variant_type_from_string('ai'))
+        vb = GLib.VariantBuilder.new(gi._gi.variant_type_from_string('ai'))
         vb.add_value(GLib.Variant.new_int32(-1))
         vb.add_value(GLib.Variant.new_int32(3))
         v = vb.end()
@@ -312,6 +325,28 @@ class TestGLib(unittest.TestCase):
         # string iteration
         self.assertEqual([x for x in v], ['h', 'e', 'l', 'l', 'o'])
 
+    def test_variant_split_signature(self):
+        self.assertEqual(GLib.Variant.split_signature('()'), [])
+
+        self.assertEqual(GLib.Variant.split_signature('s'), ['s'])
+
+        self.assertEqual(GLib.Variant.split_signature('as'), ['as'])
+
+        self.assertEqual(GLib.Variant.split_signature('(s)'), ['s'])
+
+        self.assertEqual(GLib.Variant.split_signature('(iso)'), ['i', 's', 'o'])
+
+        self.assertEqual(GLib.Variant.split_signature('(s(ss)i(ii))'), 
+                ['s', '(ss)', 'i', '(ii)'])
+
+        self.assertEqual(GLib.Variant.split_signature('(as)'), ['as'])
+
+        self.assertEqual(GLib.Variant.split_signature('(s(ss)iaiaasa(ii))'), 
+                ['s', '(ss)', 'i', 'ai', 'aas', 'a(ii)'])
+
+        self.assertEqual(GLib.Variant.split_signature('(a{iv}(ii)((ss)a{s(ss)}))'), 
+                ['a{iv}', '(ii)', '((ss)a{s(ss)})'])
+
 class TestPango(unittest.TestCase):
 
     def test_default_font_description(self):
@@ -344,10 +379,14 @@ class TestGdk(unittest.TestCase):
         self.assertEquals(color.red, 100)
         self.assertEquals(color.green, 200)
         self.assertEquals(color.blue, 300)
+        self.assertEquals(color, Gdk.Color(100, 200, 300))
+        self.assertNotEquals(color, Gdk.Color(1, 2, 3))
 
     def test_rgba(self):
         self.assertEquals(Gdk.RGBA, overrides.Gdk.RGBA)
         rgba = Gdk.RGBA(0.1, 0.2, 0.3, 0.4)
+        self.assertEquals(rgba, Gdk.RGBA(0.1, 0.2, 0.3, 0.4))
+        self.assertNotEquals(rgba, Gdk.RGBA(0.0, 0.2, 0.3, 0.4))
         self.assertEquals(rgba.red, 0.1)
         self.assertEquals(rgba.green, 0.2)
         self.assertEquals(rgba.blue, 0.3)
@@ -517,8 +556,8 @@ class TestGtk(unittest.TestCase):
         class SignalTest(GObject.GObject):
             __gtype_name__ = "GIOverrideSignalTest"
             __gsignals__ = {
-                "test-signal": (GObject.SIGNAL_RUN_FIRST,
-                                GObject.TYPE_NONE,
+                "test-signal": (GObject.SignalFlags.RUN_FIRST,
+                                None,
                                 []),
             }
 
@@ -690,6 +729,7 @@ class TestGtk(unittest.TestCase):
         tree_store = Gtk.TreeStore(int,
                                    'gchararray',
                                    TestGtk.TestClass,
+                                   GObject.TYPE_PYOBJECT,
                                    object,
                                    object,
                                    object,
@@ -703,24 +743,86 @@ class TestGtk(unittest.TestCase):
                                    GObject.TYPE_CHAR)
 
         parent = None
-        for i in range(100):
+        for i in range(97):
             label = 'this is child #%d' % i
             testobj = TestGtk.TestClass(self, i, label)
             parent = tree_store.append(parent, (i,
                                                 label,
                                                 testobj,
+                                                testobj,
                                                 test_pyobj,
                                                 test_pydict,
                                                 test_pylist,
                                                 i % 2,
                                                 bool(i % 2),
                                                 i,
-                                                9223372036854775808,
-                                                -9223372036854775808,
+                                                GObject.G_MAXULONG,
+                                                GObject.G_MININT64,
                                                 0xffffffffffffffff,
                                                 254,
                                                 _bytes('a')
                                                 ))
+        # test set
+        parent = tree_store.append(parent)
+        i = 97
+        label = 'this is child #%d' % i
+        testobj = TestGtk.TestClass(self, i, label)
+        tree_store.set(parent, 0, i,
+                               2, testobj,
+                               1, label,
+                               3, testobj,
+                               4, test_pyobj,
+                               5, test_pydict,
+                               6, test_pylist,
+                               7, i % 2,
+                               8, bool(i % 2),
+                               9, i,
+                               10, GObject.G_MAXULONG,
+                               11, GObject.G_MININT64,
+                               12, 0xffffffffffffffff,
+                               13, 254,
+                               14, _bytes('a'))
+
+        parent = tree_store.append(parent)
+        i = 98
+        label = 'this is child #%d' % i
+        testobj = TestGtk.TestClass(self, i, label)
+        tree_store.set(parent, {0: i,
+                                2: testobj,
+                                1: label,
+                                3: testobj,
+                                4: test_pyobj,
+                                5: test_pydict,
+                                6: test_pylist,
+                                7: i % 2,
+                                8: bool(i % 2),
+                                9: i,
+                                10: GObject.G_MAXULONG,
+                                11: GObject.G_MININT64,
+                                12: 0xffffffffffffffff,
+                                13: 254,
+                                14: _bytes('a')})
+
+        parent = tree_store.append(parent)
+        i = 99
+        label = 'this is child #%d' % i
+        testobj = TestGtk.TestClass(self, i, label)
+        tree_store.set(parent, (0, 2, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14),
+                               (i,
+                                testobj,
+                                label,
+                                testobj,
+                                test_pyobj,
+                                test_pydict,
+                                test_pylist,
+                                i % 2,
+                                bool(i % 2),
+                                i,
+                                GObject.G_MAXULONG,
+                                GObject.G_MININT64,
+                                0xffffffffffffffff,
+                                254,
+                                _bytes('a')))
 
         # len gets the number of children in the root node
         # since we kept appending to the previous node
@@ -737,31 +839,33 @@ class TestGtk(unittest.TestCase):
            s = tree_store.get_value(treeiter, 1)
            obj = tree_store.get_value(treeiter, 2)
            obj.check(i, s)
+           obj2 = tree_store.get_value(treeiter, 3)
+           self.assertEquals(obj, obj2);
 
-           pyobj = tree_store.get_value(treeiter, 3)
+           pyobj = tree_store.get_value(treeiter, 4)
            self.assertEquals(pyobj, test_pyobj)
-           pydict = tree_store.get_value(treeiter, 4)
+           pydict = tree_store.get_value(treeiter, 5)
            self.assertEquals(pydict, test_pydict)
-           pylist = tree_store.get_value(treeiter, 5)
+           pylist = tree_store.get_value(treeiter, 6)
            self.assertEquals(pylist, test_pylist)
 
-           bool_1 = tree_store.get_value(treeiter, 6)
-           bool_2 = tree_store.get_value(treeiter, 7)
+           bool_1 = tree_store.get_value(treeiter, 7)
+           bool_2 = tree_store.get_value(treeiter, 8)
            self.assertEquals(bool_1, bool_2)
            self.assertTrue(isinstance(bool_1, bool))
            self.assertTrue(isinstance(bool_2, bool))
 
-           uint_ = tree_store.get_value(treeiter, 8)
+           uint_ = tree_store.get_value(treeiter, 9)
            self.assertEquals(uint_, i)
-           ulong_ = tree_store.get_value(treeiter, 9)
-           self.assertEquals(ulong_, 9223372036854775808)
-           int64_ = tree_store.get_value(treeiter, 10)
-           self.assertEquals(int64_, -9223372036854775808)
-           uint64_ = tree_store.get_value(treeiter, 11)
+           ulong_ = tree_store.get_value(treeiter, 10)
+           self.assertEquals(ulong_, GObject.G_MAXULONG)
+           int64_ = tree_store.get_value(treeiter, 11)
+           self.assertEquals(int64_, GObject.G_MININT64)
+           uint64_ = tree_store.get_value(treeiter, 12)
            self.assertEquals(uint64_, 0xffffffffffffffff)
-           uchar_ = tree_store.get_value(treeiter, 12)
+           uchar_ = tree_store.get_value(treeiter, 13)
            self.assertEquals(ord(uchar_), 254)
-           char_ = tree_store.get_value(treeiter, 13)
+           char_ = tree_store.get_value(treeiter, 14)
            self.assertEquals(char_, 'a')
 
            parent = treeiter
@@ -773,21 +877,14 @@ class TestGtk(unittest.TestCase):
         class TestPyObject(object):
             pass
 
-        class TestPyGObject(GObject.Object):
-            __gtype_name__ = 'TestPyGObject'
-            def __init__(self, i):
-                GObject.Object.__init__(self)
-                self.sentinal = i + 5
-
         test_pyobj = TestPyObject()
         test_pydict = {1:1, "2":2, "3":"3"}
         test_pylist = [1,"2", "3"]
 
-        list_store = Gtk.ListStore(int, str, 'GIOverrideTreeAPITest', object, object, object, bool, bool, object)
-        for i in range(93):
+        list_store = Gtk.ListStore(int, str, 'GIOverrideTreeAPITest', object, object, object, bool, bool)
+        for i in range(1, 93):
             label = 'this is row #%d' % i
             testobj = TestGtk.TestClass(self, i, label)
-            testpygobj = TestPyGObject(i)
             parent = list_store.append((i,
                                         label,
                                         testobj,
@@ -795,8 +892,7 @@ class TestGtk(unittest.TestCase):
                                         test_pydict,
                                         test_pylist,
                                         i % 2,
-                                        bool(i % 2),
-                                        testpygobj))
+                                        bool(i % 2)))
 
         i = 93
         label = _unicode('this is row #93')
@@ -809,7 +905,17 @@ class TestGtk(unittest.TestCase):
         list_store.set_value(treeiter, 5, test_pylist)
         list_store.set_value(treeiter, 6, 1)
         list_store.set_value(treeiter, 7, True)
-        list_store.set_value(treeiter, 8, TestPyGObject(i))
+
+        # test prepend
+        label = 'this is row #0'
+        list_store.prepend((0,
+                            label,
+                            TestGtk.TestClass(self, 0, label),
+                            test_pyobj,
+                            test_pydict,
+                            test_pylist,
+                            0,
+                            False))
 
         # test automatic unicode->str conversion
         i = 94
@@ -821,8 +927,7 @@ class TestGtk(unittest.TestCase):
                                       test_pydict,
                                       test_pylist,
                                       0,
-                                      False,
-                                      TestPyGObject(i)))
+                                      False))
 
         # add sorted items out of order to test insert* apis
         # also test sending in None to not set a column
@@ -835,8 +940,7 @@ class TestGtk(unittest.TestCase):
                                       None,
                                       test_pylist,
                                       1,
-                                      None,
-                                      TestPyGObject(i)))
+                                      None))
 
         list_store.set_value(treeiter, 0, i)
         list_store.set_value(treeiter, 1, label)
@@ -854,8 +958,7 @@ class TestGtk(unittest.TestCase):
                                  test_pydict,
                                  test_pylist,
                                  1,
-                                 True,
-                                 TestPyGObject(i)))
+                                 True))
 
         i = 96
         label = 'this is row #96'
@@ -866,8 +969,7 @@ class TestGtk(unittest.TestCase):
                                             test_pydict,
                                             test_pylist,
                                             0,
-                                            False,
-                                            TestPyGObject(i)))
+                                            False))
 
         i = 98
         label = 'this is row #98'
@@ -878,8 +980,7 @@ class TestGtk(unittest.TestCase):
                                            test_pydict,
                                            test_pylist,
                                            0,
-                                           False,
-                                           TestPyGObject(i)))
+                                           False))
 
 
         i = 95
@@ -891,10 +992,44 @@ class TestGtk(unittest.TestCase):
                                test_pydict,
                                test_pylist,
                                1,
-                               True,
-                               TestPyGObject(i)))
+                               True))
 
-        self.assertEquals(len(list_store), 100)
+        i = 100
+        label = 'this is row #100'
+        treeiter = list_store.append()
+        list_store.set(treeiter, 1, label,
+                                 0, i,
+                                 2, TestGtk.TestClass(self, i, label),
+                                 3, test_pyobj,
+                                 4, test_pydict,
+                                 5, test_pylist,
+                                 6, 0,
+                                 7, False)
+        i = 101
+        label = 'this is row #101'
+        treeiter = list_store.append()
+        list_store.set(treeiter, {1: label,
+                                  0: i,
+                                  2: TestGtk.TestClass(self, i, label),
+                                  3: test_pyobj,
+                                  4: test_pydict,
+                                  5: test_pylist,
+                                  6: 1,
+                                  7: True})
+        i = 102
+        label = 'this is row #102'
+        treeiter = list_store.append()
+        list_store.set(treeiter, (1, 0, 2, 3, 4, 5, 6, 7),
+                                  (label,
+                                   i,
+                                   TestGtk.TestClass(self, i, label),
+                                   test_pyobj,
+                                   test_pydict,
+                                   test_pylist,
+                                   0,
+                                   False))
+
+        self.assertEquals(len(list_store), 103)
 
         # walk the list to see if the values were stored correctly
         i = 0
@@ -921,14 +1056,11 @@ class TestGtk(unittest.TestCase):
             self.assertTrue(isinstance(bool_1, bool))
             self.assertTrue(isinstance(bool_2, bool))
 
-            pygobj = list_store.get_value(treeiter, 8)
-            self.assertEquals(pygobj.sentinal, i + 5)
-
             treeiter = list_store.iter_next(treeiter)
 
             counter += 1
 
-        self.assertEquals(i, 99)
+        self.assertEquals(i, 102)
 
     def test_tree_path(self):
         p1 = Gtk.TreePath()
@@ -1279,7 +1411,7 @@ class TestGtk(unittest.TestCase):
         self.assertEquals('Hello World', text)
 
     def test_label(self):
-        label = Gtk.Label('Hello')
+        label = Gtk.Label(label='Hello')
         self.assertEquals(label.get_text(), 'Hello')
 
     def adjustment_check(self, adjustment, value=0.0, lower=0.0, upper=0.0,
@@ -1330,7 +1462,7 @@ class TestGtk(unittest.TestCase):
         self.assertEquals(table.get_size(), (2,3))
         self.assertEquals(table.get_homogeneous(), True)
 
-        label = Gtk.Label('Hello')
+        label = Gtk.Label(label='Hello')
         table.attach(label, 0, 1, 0, 1)
         self.assertEquals(label, table.get_children()[0])
 
@@ -1383,7 +1515,9 @@ class TestGtk(unittest.TestCase):
 class TestGio(unittest.TestCase):
     def setUp(self):
         os.environ['GSETTINGS_BACKEND'] = 'memory'
-        os.environ['GSETTINGS_SCHEMA_DIR'] = os.path.dirname(__file__)
+        # support a separate build tree, so look in build dir first
+        os.environ['GSETTINGS_SCHEMA_DIR'] = os.environ.get('TESTS_BUILDDIR', 
+                os.path.dirname(__file__))
         self.settings = Gio.Settings('org.gnome.test')
         # we change the values in the tests, so set them to predictable start
         # value
@@ -1474,11 +1608,3 @@ class TestGio(unittest.TestCase):
         self.assertEqual(len(empty), 0)
         self.assertEqual(bool(empty), True)
         self.assertEqual(empty.keys(), [])
-
-    def test_closures(self):
-        # make sure this doesn't crash
-        def fake_cb(*args):
-            pass
-
-        ag = Gtk.AccelGroup()
-        ag.connect(Gdk.KEY_l, Gdk.ModifierType.CONTROL_MASK, 0, fake_cb)
